"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CopyAssets = void 0;
const fs = require("fs");
const path = require("path");
const typedoc_1 = require("typedoc");
/**
 * Copies the assets found in comments to the output assets folder.
 */
class CopyAssets {
    constructor() {
        /** Maps a reflection to a source file it was created from. */
        this._reflectionSourcePaths = new Map();
        /** Maps a reflection to a target file it will be located at. */
        this._reflectionTargetPaths = new Map();
        /** Maps an image absolute path to a relative path in the output. */
        this._referencePathMaps = new Map();
        /** Path that we already used. */
        this._usedOutputPaths = new Set();
        /**
         * The pattern used to find references in markdown.
         */
        this._referencePattern = /(\[.*?\]\()(.*?)(\))/g;
    }
    /**
     * Create a new RelativeIncludesConverterComponent instance.
     *
     * @param typedoc The application.
     */
    initialize(typedoc) {
        this._typedoc = typedoc;
        typedoc.converter.on(typedoc_1.Converter.EVENT_CREATE_DECLARATION, (c, r, n) => {
            const filePath = this.getFolderPaths(n, r, c);
            if (!filePath)
                return;
            this._reflectionSourcePaths.set(r, filePath);
        });
        typedoc.renderer.on(typedoc_1.Renderer.EVENT_BEGIN, () => {
            this._outFolder = this._typedoc?.options.getValue('out');
            if (!this._outFolder)
                return;
            this._outMediaFolder = path.join(this._outFolder, 'assets', 'media');
            if (!fs.existsSync(this._outMediaFolder)) {
                fs.mkdirSync(this._outMediaFolder, { recursive: true });
            }
        });
        let currentReflection = undefined;
        let currentOutputFilePath = undefined;
        typedoc.renderer.on(typedoc_1.PageEvent.BEGIN, (event) => {
            currentOutputFilePath = event.url;
            currentReflection =
                event.model instanceof typedoc_1.Reflection ? event.model : undefined;
            if (currentReflection) {
                this._reflectionTargetPaths.set(currentReflection, event.url);
            }
        });
        // typedoc.renderer.on(
        //   MarkdownEvent.PARSE,
        //   (event: MarkdownEvent) => {
        //     if (!currentOutputFilePath) return;
        //     if (!currentReflection) return;
        //     if (!this._outMediaFolder) return;
        //     const folderPaths = this._reflectionSourcePaths.get(currentReflection);
        //     if (!folderPaths) return;
        //     event.parsedText = this.processText(
        //       event.parsedText,
        //       folderPaths,
        //       currentOutputFilePath
        //     );
        //   },
        //   undefined,
        //   1 // Do it before the default
        // );
        typedoc.converter.on(typedoc_1.Converter.EVENT_RESOLVE_BEGIN, (context) => {
            const project = context.project;
            currentReflection = project;
            currentOutputFilePath =
                this._reflectionTargetPaths.get(currentReflection);
            if (!currentOutputFilePath)
                return;
            if (!currentReflection)
                return;
            if (!this._outMediaFolder)
                return;
            const folderPaths = this._reflectionSourcePaths.get(currentReflection);
            if (!folderPaths)
                return;
            if (project.readme) {
                project.readme = this.processText(project.readme, folderPaths, currentOutputFilePath);
            }
            // go through all the reflections' comments
            for (const reflection of project.getReflectionsByKind(typedoc_1.ReflectionKind.All)) {
                currentReflection = context.scope;
                currentOutputFilePath =
                    this._reflectionTargetPaths.get(currentReflection);
                if (!currentOutputFilePath)
                    continue;
                if (!currentReflection)
                    continue;
                if (!this._outMediaFolder)
                    continue;
                const folderPaths = this._reflectionSourcePaths.get(currentReflection);
                if (!folderPaths)
                    continue;
                if (reflection.comment) {
                    reflection.comment.shortText = this.processText(reflection.comment.shortText, folderPaths, currentOutputFilePath);
                    reflection.comment.text = this.processText(reflection.comment.text, folderPaths, currentOutputFilePath);
                    if (reflection.comment.returns) {
                        reflection.comment.returns = this.processText(reflection.comment.returns, folderPaths, currentOutputFilePath);
                    }
                    for (const tag of reflection.comment.tags) {
                        tag.text = this.processText(tag.text, folderPaths, currentOutputFilePath);
                    }
                }
            }
        }, undefined, 1 // Do it before the default
        );
    }
    /**
     * Get the folder path for the current item.
     *
     * @param n Node.
     * @param r Reflection.
     * @param c Context.
     * @returns The folder path for the current context item or undefined.
     */
    getFolderPaths(n, r, c) {
        if (r.parent) {
            const filePath = this.getNodeFilePath(n) ?? this.getReflectionFilePath(r);
            return filePath ? [path.dirname(filePath)] : undefined;
        }
        else {
            const result = [];
            const filePath = this.getReflectionFilePath(r);
            if (filePath) {
                result.push(path.dirname(filePath));
            }
            result.push(c.program.getCurrentDirectory());
            return result;
        }
    }
    /**
     * Get the first file the reflection was created from.
     *
     * @param reflection The reflection.
     * @returns The file path.
     */
    getReflectionFilePath(reflection) {
        if (!reflection.sources || reflection.sources.length === 0)
            return;
        return reflection.sources[0].fileName;
    }
    /**
     * Get the file the node was created from.
     *
     * @param node The node.
     * @returns The file path.
     */
    getNodeFilePath(node) {
        if (!node)
            return undefined;
        if ('fileName' in node)
            return node['fileName'];
        return this.getNodeFilePath(node.parent);
    }
    /**
     * Process the text collecting the references from it and copying to the out folder.
     *
     * @param text The text to process.
     * @param originalFolderPath Path of the parsed file.
     * @param originalFolderPaths
     * @param outputFilePath Path of the documentation file.
     * @returns The modified comment.
     */
    processText(text, originalFolderPaths, outputFilePath) {
        return text.replace(this._referencePattern, (_match, prefix, pathGroup, suffix) => {
            if (typeof pathGroup === 'string' &&
                (pathGroup.startsWith('./') || pathGroup.startsWith('../'))) {
                let referencePath;
                for (const p of originalFolderPaths) {
                    const possiblePath = path.join(p, pathGroup);
                    if (fs.existsSync(possiblePath)) {
                        referencePath = possiblePath;
                        break;
                    }
                }
                if (!referencePath) {
                    console.warn(`Missing file on relative paths: ${pathGroup}, paths tried: [${originalFolderPaths.join(', ')}]`);
                    return prefix + pathGroup + suffix;
                }
                let newPath = this._referencePathMaps.get(referencePath);
                if (!newPath) {
                    const ext = path.extname(referencePath);
                    const fileName = path.basename(referencePath, ext);
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    newPath = path.join(this._outMediaFolder, fileName + ext);
                    let index = 0;
                    while (this._usedOutputPaths.has(newPath)) {
                        newPath = path.join(
                        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                        this._outMediaFolder, `${fileName}_${index++}${ext}`);
                    }
                    this._usedOutputPaths.add(newPath);
                    this._referencePathMaps.set(referencePath, newPath);
                    // console.log(
                    //   `Copy ${referencePath} => ${newPath}, exists: ${fs.existsSync(
                    //     referencePath
                    //   )}`
                    // );
                    fs.copyFileSync(referencePath, newPath);
                }
                return (prefix +
                    path
                        .relative(
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    path.dirname(path.join(this._outFolder, outputFilePath)), 
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    newPath)
                        .replace(/\\/g, '/') +
                    suffix);
            }
            else {
                return _match;
            }
        });
    }
}
exports.CopyAssets = CopyAssets;
//# sourceMappingURL=copy-assets.js.map